// Copyright (c) 2023 ESWIN Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef ESWIN_HOST_RUNTIME_INCLUDE_TASKSCHEDULER_H_
#define ESWIN_HOST_RUNTIME_INCLUDE_TASKSCHEDULER_H_

#include <map>
#include <atomic>
#include <memory>
#include <vector>
#include <thread>
#include <string>
#include <cstdint>
#include "esMutex.h"
#include "core.h"
#include "model.h"

#define MAX_NUM_UMD_OP 1024
namespace eswin
{
class TaskScheduler
{
 public:
    explicit TaskScheduler(npuContext_t *ctx);
    ~TaskScheduler();
    void startSchedule();
    void stopSchedule();
    priv::esModel *getModel();
    npuTask_t *getNpuTask();
    static int32_t complete(void *arg);

 private:
    void schedulerLoop();
    void prepareTask();
    int32_t submitTask();
    int32_t umdSubmitTask(uint32_t umdSubmodelId);
    int32_t kmdSubmitTask(uint16_t kmdSubmodelId, const kmd_dump_info_t *dumpInfo, bool isDump);
    void waitTaskComplete();
    int32_t handleCompletedTask();

    // helper function
    bool schedulerExecute();
    void handleFailure(const std::string &message);
    static int32_t writeBufToFile(const char *path, void *buf, uint32_t size);
    static int32_t dumpUmdOp(opTaskDesc_t *opTask, bool isOutput);

    // Attributes
 private:
    std::atomic<bool> mRunning;
    std::thread mScheThread;

    /**
     * used by the task, restored to mDependecySave before new task is executed.
     */
    uint8_t mUmdDependecy[MAX_NUM_UMD_OP];  // 1024
    npuTask_t *mTask;
    priv::esModel *mModel;
    npuContext_t *mCtx;
    EsMutex esMutex;
    std::atomic<bool> mIsCompleted;
};
}  // namespace eswin

#endif  // ESWIN_HOST_RUNTIME_INCLUDE_TASKSCHEDULER_H_
